clear
close all
%% generate data
% generates n-dimensional 2-class data with normal distributions
dim = 3; %dimension of the samples
numSamples = 5000; %total number of samples
ratio = 1/1; %pos/neg samples ratio
mean1 = [1 2.5 2]; %mean of the first gaussian (positive samples)
mean2 = [0 0 2]; %mean of the second gaussian (negative samples)
variance = 1; %the variance is equal for the both gaussians

numPosSamples = round(numSamples*(ratio/(1+ratio)));% number of positive samples
numNegSamples = round(numSamples/(ratio+1));% number of negative samples

%data is a structure with samples and class assignment
samples = repmat(mean1,numPosSamples,1)+variance*randn(numPosSamples,dim);
data.samples = [samples; repmat(mean2,numNegSamples,1)+variance*randn(numNegSamples,dim)];
data.class = [ones(numPosSamples,1); zeros(numNegSamples,1)];

%plot generated data
plot3(data.samples(1:numPosSamples,1),data.samples(1:numPosSamples,2),data.samples(1:numPosSamples,3),'r.');
hold on
plot3(data.samples(numPosSamples+1:end,1),data.samples(numPosSamples+1:end,2),data.samples(numPosSamples+1:end,3),'b.');
legend('Positive Data', 'Negative Data');
grid on

%select the training data (only for bayes classifier)
trainingSize = 100;
perm = randPerm(numSamples);
trainingData.samples = data.samples(perm(1:trainingSize),:);
trainingData.class = data.class(perm(1:trainingSize),:);

testData.samples = data.samples(perm(trainingSize+1:end),:);
testData.class = data.class(perm(trainingSize+1:end),:);

%% ROC construction and AUC computing
nb = NaiveBayes.fit(trainingData.samples,trainingData.class);

c1output = sumClass(testData.samples);
c2output = nb.posterior(testData.samples);
c2output = c2output(:,2);

ROC1 = ROCconstr(c1output,testData.class); % ROC construction
AUC1 = AUCcomp(ROC1); %AUC computing
ROC2 = ROCconstr(c2output,testData.class);
AUC2 = AUCcomp(ROC2);

%ROC plotting
figure
plot(1-ROC1.specificity,ROC1.sensitivity,'g');
hold on
plot(1-ROC2.specificity,ROC2.sensitivity,'k');
leg1 = sprintf('ROC of the %s, AUC = %0.3f','Sum classifier',AUC1);
leg2 = sprintf('ROC of the %s, AUC = %0.3f','Naive Bayes classifier',AUC2);
legend(leg1,leg2);
xlabel('FPR (1-Specificity)');
ylabel('TPR (Sensitivity)');
